@@ -1,17 +1,58 @@
 package Perl::Lint;
-use 5.008005;
+use 5.010001;
 use strict;
 use warnings;
 use Carp ();
 use Compiler::Lexer;
 use Module::Pluggable;
-use parent "Exporter";
-our @EXPORT_OK = qw/lint/;
+use Module::Load;
 
-our $VERSION = "0.01_01";
+our $VERSION = "0.01_02";
+
+sub new {
+    my ($class, $args) = @_;
+
+    my @ignores;
+
+    if (my $ignores = $args->{ignore}) {
+        if (ref $ignores ne 'ARRAY') {
+            Carp::croak "`ignore` must be array reference";
+        }
+
+        push @ignores, map {"Perl::Lint::Policy::$_"} @$ignores;
+    }
+
+    if (my $filters = $args->{filter}) {
+        if (ref $filters ne 'ARRAY') {
+            Carp::croak "`filter` must be array reference";
+        }
+
+        for my $filter (@$filters) {
+            my $filter_package = "Perl::Lint::Filter::$filter";
+            load $filter_package;
+
+            push @ignores, map {"Perl::Lint::Policy::$_"} @{$filter_package->filter};
+        }
+    }
+
+    Module::Pluggable->import(
+        search_path => 'Perl::Lint::Policy',
+        require     => 1,
+        inner       => 0,
+        except      => [@ignores],
+    );
+    my @site_policies = plugins(); # Exported by Module::Pluggable
+
+    # TODO add mechanism to add extend policies
+
+    bless {
+        args => $args,
+        site_policies => \@site_policies,
+    }, $class;
+}
 
 sub lint {
-    my ($files, $args) = @_;
+    my ($self, $files) = @_;
 
     my @files = ($files); # when scalar value
     if (my $ref = ref $files) {
@@ -21,38 +62,40 @@ sub lint {
         @files = @$files;
     }
 
-    # TODO to be more pluggable!
-    Module::Pluggable->import(
-        search_path => 'Perl::Lint::Policy',
-        require     => 1,
-        inner       => 0
-    );
-    my @site_policy_names = plugins(); # Exported by Module::Pluggable
-
     my @violations;
     for my $file (@files) {
-        my ($tokens, $src) = _tokenize($file);
+        open my $fh, '<', $file or die "Cannnot open $file: $!";
+        my $src = do { local $/; <$fh> };
 
-        for my $policy (@site_policy_names) {
-            push @violations, @{$policy->evaluate($file, $tokens, $src, $args)};
-        }
+        push @violations, @{$self->_lint($src, $file)};
     }
 
     return \@violations;
 }
 
-sub _tokenize {
-    my ($file) = @_;
-    open my $fh, '<', $file or die "Cannnot open $file: $!";
-    my $src = do { local $/; <$fh> };
+sub lint_string {
+    my ($self, $src) = @_;
+    return $self->_lint($src);
+}
+
+sub _lint {
+    my ($self, $src, $file) = @_;
+
+    my $args = $self->{args};
 
     my $lexer = Compiler::Lexer->new($file);
     my $tokens = $lexer->tokenize($src);
 
-    return ($tokens, $src);
+    my @violations;
+    for my $policy (@{$self->{site_policies}}) {
+        push @violations, @{$policy->evaluate($file, $tokens, $src, $args)};
+    }
+
+    return \@violations;
 }
 
 1;
+
 __END__
 
 =encoding utf-8
@@ -63,10 +106,11 @@ Perl::Lint - Yet Another Perl Source Code Linter
 
 =head1 SYNOPSIS
 
-    use Perl::Lint qw(lint);
+    use Perl::Lint;
 
+    my $linter = Perl::Lint->new;
     my $target_files = [qw(foo/bar.pl buz.pm)];
-    my $violations   = lint($target_files);
+    my $violations   = $linter->lint($target_files);
 
 =head1 DESCRIPTION
 
@@ -82,11 +126,11 @@ Development of this module aims to create a fast and flexible static analyzer fo
 
 Please see also L<http://news.perlfoundation.org/2014/03/grant-proposal-perllint---yet.html>.
 
-=head1 FUNCTIONS
+=head1 METHODS
 
 =over 4
 
-=item * C<lint($target_files:SCALAR or ARRAYREF, $args:HASHREF)>
+=item * C<< $linter->lint($target_files:SCALAR or ARRAYREF, $args:HASHREF) >>
 
 C<lint> checks the violations of target files. It can export.
 On default, this function checks the all of policies that are in C<Perl::Lint::Policy::*>.
